﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="DcDomainCodeEntity.cs" company="DataCommunication">
//   DCFramework
// </copyright>
// <summary>
//   The dc dummy entity.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace DCFrameworkSolution.Xafari.Module.BusinessObjects
{
    using System.ComponentModel;

    using DCFrameworkSolution.Core.BusinessObjectInterfaces;

    using DevExpress.ExpressApp;
    using DevExpress.ExpressApp.DC;

    /// <summary>
    /// The dc dummy entity.
    /// </summary>
    [DomainComponent]
    public class DcDomainCodeEntity : IXafEntityObject, IObjectSpaceLink, INotifyPropertyChanged, IDcCode
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="DcDomainCodeEntity"/> class.
        /// </summary>
        /// <param name="code">
        /// The code.
        /// </param>
        public DcDomainCodeEntity(string code)
        {
            this.Code = code;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="DcDomainCodeEntity"/> class.
        /// </summary>
        public DcDomainCodeEntity()
        {
        }

        /// <summary>
        /// Gets or sets the id.
        /// </summary>
        [Browsable(false)]
        public int Id { get; protected set; }

        /// <summary>
        /// Gets or sets the code.
        /// </summary>
        public string Code { get; set; }

        #region IObjectSpaceLink members (see http://help.devexpress.com/#Xaf/clsDevExpressExpressAppIObjectSpaceLinktopic)

        /// <summary>
        /// Gets or sets the object space.
        /// </summary>
        IObjectSpace IObjectSpaceLink.ObjectSpace { get; set; }

        #endregion

        #region IXafEntityObject members (see http://help.devexpress.com/#Xaf/clsDevExpressExpressAppIXafEntityObjecttopic)

        /// <summary>
        /// The on created.
        /// </summary>
        void IXafEntityObject.OnCreated()
        {
        }

        /// <summary>
        /// The on loaded.
        /// </summary>
        void IXafEntityObject.OnLoaded()
        {
        }

        /// <summary>
        /// The on saving.
        /// </summary>
        void IXafEntityObject.OnSaving()
        {
        }

        #endregion

        #region INotifyPropertyChanged members (see http://msdn.microsoft.com/en-us/library/system.componentmodel.inotifypropertychanged(v=vs.110).aspx)

        /// <summary>
        /// The on property changed.
        /// </summary>
        /// <param name="propertyName">
        /// The property name.
        /// </param>
        private void OnPropertyChanged(string propertyName)
        {
            this.PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        /// <summary>
        /// The property changed.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        #endregion
    }
}