﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DevExpress.Data.Filtering;
using DevExpress.ExpressApp;
using DevExpress.ExpressApp.Actions;
using DevExpress.ExpressApp.Editors;
using DevExpress.ExpressApp.Layout;
using DevExpress.ExpressApp.Model.NodeGenerators;
using DevExpress.ExpressApp.SystemModule;
using DevExpress.ExpressApp.Templates;
using DevExpress.ExpressApp.Utils;
using DevExpress.Persistent.Base;
using DevExpress.Persistent.Validation;

namespace DCFrameworkSolution.Xafari.Module.Controllers
{
    using System.Collections;
    using System.Data.Entity.Core.Objects;

    using DCFrameworkSolution.Core.BusinessObjectInterfaces;
    using DCFrameworkSolution.Core.BusinessObjects;
    using DCFrameworkSolution.Core.Providers;
    using DCFrameworkSolution.Xafari.Module.BusinessObjects;

    using global::Xafari;
    using global::Xafari.Model;
    using global::Xafari.Actions;

    // For more typical usage scenarios, be sure to check out https://documentation.devexpress.com/eXpressAppFramework/clsDevExpressExpressAppViewControllertopic.aspx.
    public partial class UndoWizardForDeletedEntitiesViewController : ViewController
    {
        private DetailView detailView;

        public UndoWizardForDeletedEntitiesViewController()
        {
            InitializeComponent();
            
            this.TargetViewType = ViewType.DetailView;
            this.TargetObjectType = typeof(IDcSoftDelete);

            this.saDeletedUndo.TargetObjectType = typeof(IDcSoftDelete);
            this.saDeletedCancel.TargetObjectType = typeof(IDcSoftDelete);

            this.saDeletedUndo.Active.SetItemValue("NotDeleted", false);
            this.saDeletedCancel.Active.SetItemValue("NotDeleted", false);

            this.saDeletedUndo.Model.Index = 0;
            this.saDeletedCancel.Model.Index = 1;
        }

        protected override void OnActivated()
        {
            base.OnActivated();
            
            this.detailView = this.View as DetailView;
            if ((this.detailView?.CurrentObject as IDcSoftDelete) == null
                || !((IDcSoftDelete)this.detailView?.CurrentObject).IsDeleted)
            {
                return;
            }

            this.detailView.ViewEditMode = ViewEditMode.View;
            this.detailView.AllowEdit.SetItemValue("IsDeleted", false);
            this.detailView.AllowDelete.SetItemValue("IsDeleted", false);
            this.detailView.AllowNew.SetItemValue("IsDeleted", false);
            this.detailView.Tag = "NoJS";

            this.Frame.GetController<RefreshController>().RefreshAction.Active.SetItemValue("IsDeleted", false);

            this.saDeletedUndo.Active.RemoveItem("NotDeleted");
            this.saDeletedCancel.Active.RemoveItem("NotDeleted");

            this.saDeletedUndo.Execute += this.SaDeletedUndoExecute;
            this.saDeletedCancel.Execute += this.SaDeletedCancelExecute;
        }

        protected override void OnDeactivated()
        {
            if (this.saDeletedUndo != null)
            {
                this.saDeletedUndo.Execute -= this.SaDeletedUndoExecute;
            }

            if (this.saDeletedCancel != null)
            {
                this.saDeletedCancel.Execute -= this.SaDeletedCancelExecute;
            }

            base.OnDeactivated();
        }

        private void SaDeletedUndoExecute(object sender, SimpleActionExecuteEventArgs e)
        {
            var currentObject = this.detailView.CurrentObject;
            var currentObjectType = ObjectContext.GetObjectType(currentObject.GetType());
                
            if (!this.UndoActionCheckCodeUnique(currentObjectType, currentObject))
            {
                this.SetupWizardShowViewParameters(e.ShowViewParameters, this.Application.Model.Xafari().Wizards.GetWizard("waUndoDeleted"));

                return;
            }

            this.UndoActionExecute(currentObjectType, currentObject);

            this.Frame.GetController<ModificationsController>().CancelAction.Active.RemoveItem("EditMode");
            this.Frame.GetController<ModificationsController>().CancelAction.DoExecute();
        }

        private void SaDeletedCancelExecute(object sender, SimpleActionExecuteEventArgs e)
        {
            this.Frame.GetController<ModificationsController>().CancelAction.Active.RemoveItem("EditMode");
            this.Frame.GetController<ModificationsController>().CancelAction.DoExecute();
        }

        private bool UndoActionCheckCodeUnique(Type objectType, object objectEntity)
        {
            using (var objectSpace = this.Application.CreateObjectSpace())
            {
                var localObject = objectEntity as IDcCode;
                if (localObject == null)
                {
                    return true;
                }

                var results = objectSpace.GetObjects(
                    objectType,
                    CriteriaOperator.Parse($"StartsWith(Code, '{localObject.Code}')"));

                return !(results?.Count > 0);
            }
        }

        private void UndoActionExecute(Type objectType, object objectEntity)
        {
            var dcSoftDelete = objectEntity as IDcSoftDelete;
            if (dcSoftDelete == null || !dcSoftDelete.IsDeleted || objectType == null)
            {
                throw new ArgumentException("UndoActionExecute Argument Exception");
            }

            try
            {
                var typeInfo = this.ObjectSpace.TypesInfo.FindTypeInfo(objectType);
                foreach (var memberInfo in typeInfo.Members.Where(x => x.IsAssociation))
                {
                    if (memberInfo.AssociatedMemberInfo.IsList)
                    {
                        continue;
                    }

                    var referenceMember = memberInfo.GetValue(objectEntity) as IDcSoftDelete;
                    if (!memberInfo.IsList && referenceMember != null && referenceMember.IsDeleted)
                    {
                        referenceMember.IsDeleted = false;
                    }

                    var collectionMember = memberInfo.GetValue(objectEntity) as ICollection;
                    if (!memberInfo.IsList || collectionMember == null)
                    {
                        continue;
                    }

                    foreach (var childReferenceMember in collectionMember.Cast<object>().OfType<IDcSoftDelete>().Where(childReferenceMember => childReferenceMember.IsDeleted))
                    {
                        childReferenceMember.IsDeleted = false;
                    }
                }
            }
            catch (Exception exp)
            {
                throw new InvalidOperationException("UndoActionExecute Operation Exception", exp);
            }
            
            dcSoftDelete.IsDeleted = false;
            this.ObjectSpace.CommitChanges();
        }

        private void SetupWizardShowViewParameters(ShowViewParameters showViewParameters, IModelWizard wizard)
        {
            var objectSpace = this.Application.CreateObjectSpace();
            var dummyObject = objectSpace.CreateObject<DcDomainCodeEntity>();
            dummyObject.Code = ((this.View as DetailView)?.CurrentObject as IDcCode)?.Code;

            var controller = XafariModule.CreateWizardDialogController(this.Application);
            controller.Model = wizard;

            controller.AcceptAction.Active.SetItemValue("test",true);

            controller.CurrentObject = (object)dummyObject;
            controller.Accepting += this.UndoDeletedWizardOnAccepting;
            controller.CustomNextStep += this.ControllerOnCustomNextStep;

            var createdView = this.Application.CreateDetailView(objectSpace, wizard[0].View, true, dummyObject);
            createdView.ViewEditMode = ViewEditMode.Edit;
            createdView.Tag = "NoJS";

            showViewParameters.Controllers.AddRange(new[] { controller });
            showViewParameters.Context = wizard.TemplateContext;
            showViewParameters.CreatedView = createdView;
            showViewParameters.TargetWindow = TargetWindow.NewModalWindow;
            showViewParameters.NewWindowTarget = NewWindowTarget.Default;
        }

        private void ControllerOnCustomNextStep(object sender, CustomNextStepEventArgs e)
        {
            if (e.CurrentStep.Index == 1)
            {
                if (this.UndoActionCheckCodeUnique(
                    ObjectContext.GetObjectType(this.detailView.CurrentObject.GetType()),
                    e.WizardObject<DcDomainCodeEntity>()))
                {
                    e.NextStep = e.CurrentStep.Wizard[2];
                }
                else
                {
                    e.NextStep = e.CurrentStep.Wizard[0];
                }
            }
        }

        private void UndoDeletedWizardOnAccepting(object sender, DialogControllerAcceptingEventArgs e)
        {
            var wizardDialogController = sender as WizardDialogController;
            if (wizardDialogController != null)
            {
                wizardDialogController.Accepting -= this.UndoDeletedWizardOnAccepting;
            }

            var currentObject = this.detailView.CurrentObject;
            var currentObjectType = ObjectContext.GetObjectType(currentObject.GetType());

            var dummyObject = e.AcceptActionArgs.CurrentObject as DcDomainCodeEntity;
            if (dummyObject != null)
            {
                var dcCode = currentObject as IDcCode;
                if (dcCode != null)
                {
                    dcCode.Code = dummyObject.Code;
                }
            }

            this.UndoActionExecute(currentObjectType, currentObject);

            this.Frame.GetController<ModificationsController>().CancelAction.Active.RemoveItem("EditMode");
            this.Frame.GetController<ModificationsController>().CancelAction.DoExecute();
        }
    }
}
